<?php

declare(strict_types=1);

namespace Drupal\phone_international\Commands;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drush\Commands\DrushCommands;
use Drush\Attributes as CLI;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\Filesystem\Filesystem;

/**
 * A Drush command file.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://git.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://git.drupalcode.org/devel/tree/drush.services.yml
 */
class PhoneInternationalCommands extends DrushCommands {

  /**
   * {@inheritdoc}
   */
  public function __construct(protected LibraryDiscoveryInterface $libraryDiscovery) {}

  /**
   * Download and install the Phone International plugin.
   */
  #[CLI\Command(name: 'phone_international:plugin', aliases: ['piplugin', 'pi-plugin'])]
  public function plugin($path = ''): void {
    $fs = new Filesystem();

    if (empty($path)) {
      $path = DRUPAL_ROOT . '/libraries/intl-tel-input';
    }

    // Create a path if it doesn't exist
    // Exit with a message otherwise.
    if (!$fs->exists($path)) {
      $fs->mkdir($path);
    }
    else {
      $this->logger()->notice(dt('International Telephone Input is already present at @path. No download required.', ['@path' => $path]));
      return;
    }

    // Load the phone_international defined library.
    if ($phone_international_library = $this->libraryDiscovery->getLibraryByName('phone_international', 'phone_international_general')) {
      // Download the file.
      $client = new Client();
      $destination = tempnam(sys_get_temp_dir(), 'intl-tel-input-tmp');
      try {
        $client->get($phone_international_library['remote'] . '/archive/master.zip', ['sink' => $destination]);
      }
      catch (RequestException $e) {
        // Remove the directory.
        $fs->remove($path);
        $this->logger()->error(dt('Drush was unable to download the phone international library from @remote. @exception', [
          '@remote' => $phone_international_library['remote'] . '/archive/master.zip',
          '@exception' => $e->getMessage(),
        ]));
        return;
      }

      // Move the downloaded file.
      $fs->rename($destination, $path . '/intl-tel-input.zip');

      // Unzip the file.
      $zip = new \ZipArchive();
      $res = $zip->open($path . '/intl-tel-input.zip');
      if ($res === TRUE) {
        $zip->extractTo($path);
        $zip->close();
      }
      else {
        // Remove the directory if unzip fails and exit.
        $fs->remove($path);
        $this->logger()->error(dt('Error: unable to unzip phone international file.', []));
        return;
      }

      // Remove the downloaded zip file.
      $fs->remove($path . '/intl-tel-input.zip');

      // Move the file.
      $fs->mirror($path . '/intl-tel-input-master', $path, NULL, ['override' => TRUE]);
      $fs->remove($path . '/intl-tel-input-master');

      // Success.
      $this->logger()->success(dt('The phone international library has been successfully downloaded to @path.', [
        '@path' => $path,
      ]));
    }
    else {
      $this->logger()->error(dt('Drush was unable to load the phone international library'));
    }
  }

}
