<?php

declare(strict_types=1);

namespace Drupal\phone_international\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Tel;

/**
 * Provides a phone_international form.
 *
 * Usage example:
 *
 * Note: By default, geolocation is not enabled.
 *
 * @code
 * $form['phone'] = [
 *   '#type' => 'phone_international',
 *   '#title' => $this->t('International Phone'),
 * ];
 * @endcode
 *
 * Example of setting a default country, with a limited list of countries
 * and no geolocation (geolocation is off by default).
 *
 * @code
 * $form['phone'] = [
 *   '#type' => 'phone_international',
 *   '#title' => $this->t('International Phone'),
 *   '#country' => 'PT',
 *   '#countries' => 'include',
 *   '#exclude_countries' => ['AU', AT, 'PT'],
 * ];
 * @endcode
 *
 * Example of having geolocation switched on, excluding
 * certain countries from the list, and setting preferred
 * countries, which will appear at the top of the list.
 *
 * @code
 * $form['phone'] = [
 *   '#type' => 'phone_international',
 *   '#title' => $this->t('International Phone'),
 *   '#geolocation' => 1,
 *   '#preferred_countries' => ['PT', 'CN'],
 *   '#countries' => 'exclude',
 *   '#exclude_countries' => ['GB', 'BH', 'US'],
 * @endcode
 *
 * @FormElement("phone_international")
 */
class PhoneInternationalElement extends Tel {

  /**
   * Form element validation handler for #type 'phone_international'.
   *
   * @param array $element
   *   A form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The $form_state from complete form.
   * @param array $complete_form
   *   Complete parent form.
   */
  public static function validateNumber(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    $value = $element['#value'];

    $form_state->setValueForElement($element, $value);

    if (!empty($value) && !\Drupal::service('phone_international.validate')->isValidNumber($value)) {
      $form_state->setError($element, t('The %name "%phone_international" is not valid.', [
        '%phone_international' => $value,
        '%name' => $element['#title'],
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    $info = parent::getInfo();
    $info['#pre_render'][] = [static::class, 'preRenderPhoneInternational'];
    $info['#element_validate'][] = [static::class, 'validateNumber'];
    return array_merge($info, [
      '#country' => '',
      '#geolocation' => 0,
      '#exclude_countries' => [],
      '#countries' => 'exclude',
      '#preferred_countries' => [],
      '#dial-code' => 0,
      '#auto_placeholder' => 'aggressive',
      '#national_number' => 1,
    ]);
  }

  /**
   * Adds tel and hidden input to phone_international element.
   *
   * @param array $element
   *   A form element.
   *
   * @return array
   *   Returns the element with the configured widget settings.
   */
  public static function preRenderPhoneInternational($element): array {
    $element['#attached']['library'][] = 'phone_international/phone_international';

    if ($element['#countries'] != 'all'
      && !empty($element['#exclude_countries'])
      && ($element['#countries'] === 'exclude'
      && in_array($element['#country'], $element['#exclude_countries']))
      || ($element['#countries'] === 'include'
      && !in_array($element['#country'], $element['#exclude_countries']))
    ) {
      $element['#country'] = reset($element['#exclude_countries']);
    }

    $element['#attributes'] = array_merge(
        $element['#attributes'] ?? [],
        [
          'placeholder' => $element['#placeholder'] ?? '',
          'class' => ['phone-international-number'],
          'data-country' => (int) $element['#geolocation'] === 1 ? 'auto' : $element['#country'],
          'data-dial-code' => (int) $element['#dial_code'],
          'data-geo' => (int) $element['#geolocation'],
          'data-preferred' => $element['#preferred_countries'] ? implode("-", $element['#preferred_countries']) : [],
          'data-auto-placeholder' => $element['#auto_placeholder'],
          'data-national-number' => (int) $element['#national_number'],
        ]
      );

    // Limit countries using either `data-only` or `data-exclude`.
    $key = ($element['#countries'] === 'include' ? 'data-only' : 'data-exclude');
    $element['#attributes'][$key] = !empty($element['#exclude_countries']) ? implode("-", $element['#exclude_countries']) : [];

    $path = base_path() . _phone_international_get_path();

    if (\Drupal::config('phone_international.settings')->get('cdn')) {
      $path = '//cdn.jsdelivr.net/npm/intl-tel-input/build';
    }

    $element['#attached']['drupalSettings']['phone_international']['path'] = $path;

    return $element;
  }

}
