<?php

declare(strict_types=1);

namespace Drupal\phone_international\Helpers;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;

/**
 * Class Validating Service.
 */
class PhoneNumberValidatingService implements PhoneNumberInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected LoggerChannelFactoryInterface $channelFactory,
  ) {}

  /**
   * Utility for international phone numbers.
   *
   * @param string $number
   *   Phone number verification and validation.
   *
   * @return bool
   *   Return number valid (TRUE) or invalid (FALSE).
   */
  public function isValidNumber(string $number): bool {
    $phoneUtil = PhoneNumberUtil::getInstance();

    try {
      $parseNumber = $phoneUtil->parse($number);
      return $phoneUtil->isValidNumber($parseNumber);
    }
    catch (NumberParseException $e) {
      $this->channelFactory->get('phone_international')->debug($e->getMessage());
      return FALSE;
    }
  }

  /**
   * Formats a phone number to E164 international format.
   *
   * @param string $number
   *   The phone number to format.
   *
   * @return string
   *   The formatted phone number or the original number if parsing fails.
   */
  public function formatNumber(string $number): string {
    $phoneUtil = PhoneNumberUtil::getInstance();

    try {
      $numberProto = $phoneUtil->parse($number);
      return $phoneUtil->format($numberProto, PhoneNumberFormat::E164);
    }
    catch (NumberParseException $e) {
      $this->channelFactory->get('phone_international')->error('Problem formatting number: @number. The error given was @error', [
        '@number' => $number,
        '@error'  => $e->getMessage(),
      ]);
      return $number;
    }
  }

}
