<?php

declare(strict_types=1);

namespace Drupal\phone_international\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Locale\CountryManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'phone_international' widget.
 */
#[FieldWidget(
  id: 'phone_international_widget',
  label: new TranslatableMarkup('Phone international field'),
  field_types: ['phone_international'],
)]
class PhoneInternationalDefaultWidget extends WidgetBase {

  /**
   * The country manager.
   *
   * @var \Drupal\Core\Locale\CountryManagerInterface
   */
  protected CountryManagerInterface $countryManager;

  /**
   * Constructs a RegionalForm object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Locale\CountryManagerInterface $country_manager
   *   The country manager.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, CountryManagerInterface $country_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->countryManager = $country_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('country_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
      'countries' => 'exclude',
      'dial_code' => FALSE,
      'exclude_countries' => [],
      'geolocation' => FALSE,
      'initial_country' => 'PT',
      'preferred_countries' => ['PT'],
      'auto_placeholder' => 'aggressive',
      'national_number' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element['value'] = $element + [
      '#type' => 'phone_international',
      '#default_value' => $items[$delta]->value ?? NULL,
      '#country' => $this->getSetting('initial_country'),
      '#dial_code' => $this->getSetting('dial_code'),
      '#geolocation' => (bool) $this->getSetting('geolocation'),
      '#exclude_countries' => $this->getSetting('exclude_countries'),
      '#countries' => $this->getSetting('countries'),
      '#preferred_countries' => $this->getSetting('preferred_countries'),
      '#auto_placeholder' => $this->getSetting('auto_placeholder'),
      '#national_number' => $this->getSetting('national_number'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $elements = [];

    $elements['auto_placeholder'] = [
      '#type' => 'select',
      '#title' => $this->t('Auto Placeholder'),
      '#options' => [
        'off' => $this->t('Off'),
        'aggressive' => $this->t('Aggressive'),
        'polite' => $this->t('Polite'),
      ],
      '#default_value' => $this->getSetting('auto_placeholder'),
      '#description' => $this->t('Set the input placeholder to an example number for the selected country, and update it if the country changes.'),
    ];

    $elements['national_number'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable national number display'),
      '#default_value' => $this->getSetting('national_number'),
    ];

    $elements['geolocation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Geolocation'),
      '#default_value' => $this->getSetting('geolocation'),
    ];

    $elements['dial_code'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Separate Dial Code'),
      '#description' => $this->t('Display the country dial code next to the selected flag.'),
      '#default_value' => $this->getSetting('dial_code'),
    ];

    $countries = $this->countryManager->getList();
    $elements['initial_country'] = [
      '#type' => 'select',
      '#title' => $this->t('Default country'),
      '#options' => $countries,
      '#default_value' => $this->getSetting('initial_country'),
      '#description' => $this->t('Specify a default selection country. If geolocation is enabled this setting will be ignored.'),
      '#states' => [
        'invisible' => [
          '[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][geolocation]"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $elements['preferred_countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Top list countries'),
      '#multiple' => TRUE,
      '#options' => $countries,
      '#default_value' => $this->getSetting('preferred_countries'),
      '#description' => $this->t('Specify the countries to appear at the top of the list. Leave it blank to follow an alphabetic order.'),
    ];

    $elements['countries'] = [
      '#type' => 'radios',
      '#title' => $this->t('List of countries'),
      '#options' => [
        'all' => $this->t('All'),
        'exclude' => $this->t('Exclude the selected countries'),
        'include' => $this->t('Include the selected countries'),
      ],
      '#default_value' => $this->getSetting('countries'),
    ];

    $elements['exclude_countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Countries'),
      '#title_display' => 'invisible',
      '#options' => $countries,
      '#multiple' => TRUE,
      '#default_value' => $this->getSetting('exclude_countries'),
      '#states' => [
        'invisible' => [
          '[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][countries]"]' => ['value' => 'all'],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $summary[] = $this->t('Auto Placeholder: @display_label', ['@display_label' => $this->getSetting('auto_placeholder')]);
    $geolocation = (int) $this->getSetting('geolocation');
    $summary[] = $this->t('Use Geolocation: @display_label', ['@display_label' => ($geolocation ? $this->t('Yes') : 'No')]);
    $national_number = (int) $this->getSetting('national_number');
    $summary[] = $this->t('Use National number display: @display_label', ['@display_label' => ($national_number ? $this->t('Yes') : 'No')]);
    $dial_code = (int) $this->getSetting('dial_code');
    $summary[] = $this->t('Enable Separate Dial Code: @display_label', ['@display_label' => ($dial_code ? $this->t('Yes') : 'No')]);

    if ($geolocation === 0 && ($this->getSetting('countries') === 'all')
      || (!empty($this->getSetting('exclude_countries')) && $this->getSetting('exclude_countries') === 'include' && in_array($this->getSetting('initial_country'), $this->getSetting('exclude_countries')))
      || (!empty($this->getSetting('exclude_countries')) && $this->getSetting('exclude_countries') === 'exclude' && !in_array($this->getSetting('initial_country'), $this->getSetting('exclude_countries')))
      ) {
      $summary[] = $this->t('Default country: @value', [
        '@value' => $this->getSetting('initial_country'),
      ]);
    }

    if (!empty($this->getSetting('preferred_countries'))) {
      $summary[] = $this->t('Top list countries: @value', [
        '@value' => implode(", ", $this->getSetting('preferred_countries')),
      ]);
    }

    if ($this->getSetting('countries') === 'exclude' && !empty($this->getSetting('exclude_countries'))) {
      $summary[] = $this->t('Excluded countries: @value', [
        '@value' => implode(", ", $this->getSetting('exclude_countries')),
      ]);
    }
    elseif ($this->getSetting('countries') === 'include' && !empty($this->getSetting('exclude_countries'))) {
      $summary[] = $this->t('Countries included: @value', [
        '@value' => implode(", ", $this->getSetting('exclude_countries')),
      ]);
    }
    else {
      $summary[] = $this->t('Show all countries');
    }

    return $summary;
  }

}
